<?php

/**
 * @file
 *   Utility functions that are not module specific.
 *
 *   Creator, Maintainer & Developer: benjamin melançon of Agaric Design Collective - http://agaricdesign.com/
 * 
 
 */


/**
 * Set a default value for an array key if that key does not exist.
 *
 * If the key does exist, compare values and return TRUE if the same
 * or the VALUE.
 * 
 * If the key does not exist and the default value is assigned, return TRUE. 
 *
 * The _au_ namespace stands for agaric utility, and are things 
 * we may reuse in so many modules we'll want to put in a helper
 * module or, better yet, in core.
 *
 * @param array
 *   An array.
 *
 * @param key
 *   A string representing an array key.
 *
 * @param default_value
 *   A mixed value to assign to this key if one does not exist.
 * @return mixed
 */
function salesforce_au_default(&$array, $key, $default_value) {
  if (!isset($array[$key])) {
    $array[$key] = $default_value;
    return TRUE;
  }
  else {
    if ($array[$key] == $default_value)  return FALSE;
    else  return $array[$key];
  }
}


/**
 * 
 */
function salesforce_au_count_values($array = array()) {
  $i = 0;
  $array = (array)$array;
  foreach ($array AS $value) {
    if ($value) $i++;
  }
  return $i;
}

/**
 * Takes a nested associative array and a simple array of levels and returns the value of the multilevel array at that position.
 *
 * In English: it lets you $name['a']['spot']['in']['a']['deep']['array'] as a list of levels, an array, rather than having to loop yourself to find info.
 *
 * See Agaric's post about this: 
 */
function salesforce_au_nested_array_value($array, $levels = array()) {
  foreach ($levels AS $level) {
    $array = $array[$level];
  }
  return $array;
}
