<?php
/**
 * @file
 * This file contains administrative functions only.  There is no performance
 * benefit to this separation until Drupal 7 but it can improve readability.
 */


function salesforce_admin_settings() {
//  this works, but it makes the tabs move only on this page
//  drupal_set_title(t('SalesForce.com Settings'));

  _salesforce_settings();

  $form['salesforce_account'] = array(
    '#type' => 'fieldset',
    '#title' => t('SalesForce.com Account'),
    '#collapsible' => true,
    '#description' => t('This information is required for this module to work'),
  );
  $form['salesforce_account']['salesforce_user'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#default_value' => variable_get('salesforce_user', NULL),
    '#required' => true,
  );

  // TODO: need to encrypt / decrypt this
  //for display see http://api.freestylesystems.co.uk/api/function/theme_password/5
  if (!variable_get('salesforce_password', NULL)) {
    $form['salesforce_account']['salesforce_password'] = array(
    '#type' => 'password',
    '#title' => t('Password'),
    '#default_value' => variable_get('salesforce_password', NULL),
    '#required' => true
    );
  } else {
    $form['salesforce_account']['salesforce_password_delete'] = array(
    '#prefix' => t('<strong>***** (TODO: setup like this until a better password saving feature can be implemented)</strong>'),
    '#type' => 'checkbox',
    '#title' => t('Delete Password'),
    '#default_value' => variable_get('salesforce_password_delete', false)
    );
  }

  $form['salesforce_account']['salesforce_client_type'] = array(
    '#type' => 'select',
    '#title' => t('Salesforce Account Type'),
    '#default_value' => variable_get('salesforce_client_type', 'Partner'),
    '#options' => array(
      'Partner' => 'Partner',
      'Enterprise' => 'Enterprise',
    ),
    '#required' => true,
  );
  return system_settings_form($form);
}


/**
 *  admin page for viewing the salesforce_logs table
 */
function salesforce_admin_page() {
  theme('add_style', drupal_get_path('module', 'salesforce') .'/salesforce.css');


// commented out titles works fine but aren't necessary, and interfere with consistent tabs

  $op = arg(3);

  if ($op && $op != 'logs') {
//    $breadcrumb = drupal_get_breadcrumb();
//    $breadcrumb[] = l(t('salesforce'), 'admin/logs/salesforce');
//    drupal_set_breadcrumb($breadcrumb);

//    drupal_set_title(t('SalesForce - %op', array('%op' => ucfirst($op) .'s')));
    switch ($op) {
      default:
        $output .= _salesforce_admin_list($op);
        break;
    }
  } else {
//    drupal_set_title(t('SalesForce'));


    $result = db_query("SELECT sid, type, type_id, message, timestamp, status FROM {salesforce_log} ORDER BY timestamp DESC");
    $total = db_num_rows($result);

    $output .= '<p>'. t('All tasks that were not completed successfully are logged and displayed here. If you have a cron job setup to use drupal\'s cron.php, these logged items will be attempted again on cron run.') .'</p>'. "\n";

    if ($total > 0) {
      // table columns (damn these big ass arrays kinda bug me)
      $cols = array(array(
      'data' => t('ID'),
      'class' => 'id'),
      t('type'),
      t('user'),
      array('data' => 'message', 'class' => 'message'),
      t('date'),
      array('data' => t('status'), 'class' => 'status'),
      array('data' => t('operations'), 'class' => 'op'));

      while ($row = db_fetch_object($result)) {
        if ($row->status) {
          $state = 'active';
        } else {
          $state = 'completed';
        }
        // this row kinda looks nasty but ya.. here it is
        $rows[] = array('class' => $state,
        'data' => array($row->sid, str_replace('_', ' ', $row->type),
        theme('username', user_load(array('uid' => $row->type_id))),
        $row->message, format_date(strtotime($row->timestamp), 'large'), $state,
        (($row->status) ?
        l(t('run'), 'admin/logs/salesforce/log/run/'. $row->sid) .' | ' .
        l(t('disable'), 'admin/logs/salesforce/log/disable/'. $row->sid, array('onclick' => 'if (confirm("'. t('are you sure you want to disable this log?') .'")) { return true; } else { return false; }')) .' | ' :
        l(t('enable'), 'admin/logs/salesforce/log/enable/'. $row->sid) .' | ') .
        l(t('delete'), 'admin/logs/salesforce/log/delete/'. $row->sid, array('onclick' => 'if (confirm("'. t('are you sure you want to delete this log?') .'")) { return true; } else { return false; }'))
        )
        );
      }

      $rows[] = array(array('class' => 'total', 'colspan' => 7, 'data' => t('<strong>Total:</strong> %total', array('%total' => $total))));

      $output .= theme('table', $cols, $rows, array('class' => 'salesforce_admin'));
    } else {
      $output .= '<p><strong>'. t('currently no logged items') .'</strong></p>' ."\n";
    }
  }

  print theme('page', $output);
}


/**
 * Called from admin/logs/salesforce/forms to manage Salesforce-exposed forms.
 */
function salesforce_admin_forms_page() {
  $output = '';
  // Return the HTML generated from the $form data structure
  $output .= drupal_get_form('salesforce_admin_forms');
  return $output;
}

/**
 * Generate main Salesforce forms administration form.
 */
function salesforce_admin_forms() {
  // dropdown:  <select>, Disabled, Enabled, with Event tracking
  $status_options = array(
//    NULL => t('<select>'),
    0 => t('Disabled'),
    1 => t('Enabled'),
    2 => t('with Event tracking'),
  );
  // @TODO: these should be defined somewhere else
  $type_options = array(
    'lead' => t('Lead'),
    'contact' => t('Contact'),
  );

  $salesforce_forms = salesforce_list_forms();
  
  if (!count($salesforce_forms)) {
    drupal_set_message(t('There are no forms currently exposed to Salesforce.'));
  }

  // Build form tree
  $form['#tree'] = TRUE;
  foreach ($salesforce_forms as $id => $sf) {
    $form[$id]['form_id'] = array(
      '#type' => 'markup',
      '#value' => $id,
    );
    $form[$id]['type'] = array(
      '#type' => 'select',
      '#default_value' => $sf['type'],
      '#options' => $type_options,
      '#disabled' => TRUE,
    );
    $form[$id]['status'] = array(
      '#type' => 'select', 
      '#default_value' => $sf['status'], 
      '#options' => $status_options,
    );
    $form[$id]['configure'] = array(
      '#value' => l(t('configure'), 'admin/logs/salesforce/forms/configure_form/'. $id));
  }
  $form['submit'] = array(
    '#type' => 'submit', 
    '#value' => t('Save forms settings'),
  );

  return $form;
}


/**
 * Theme Salesforce forms administration form submission.
 *
 * Provide a table suitable for the administration page.
 */
function theme_salesforce_admin_forms($form) {
  // Build rows
  $rows = array();
  foreach (element_children($form) as $id) {  
    $sf = &$form[$id];
// drupal_set_message('<pre>'. var_export($id,TRUE) .'</pre>');  
// drupal_set_message('<pre>'. var_export($sf,TRUE).'</pre>');
    // Only take form elements that are Salesforce form representations.
    if (is_array($sf['form_id'])) {
      // Fetch values
      $type = $sf['type']['#default_value'];
      $status = $sf['status']['#default_value'];

      // Generate Salesforce-exposed form row
      $row = array(
        array(
          'data' => drupal_render($sf['form_id']),
          'class' => 'form_id',
        ),
      );
      $row[] = drupal_render($sf['type']);
      $row[] = drupal_render($sf['status']);
      $row[] = drupal_render($sf['configure']);
      $rows[] = $row;
    }
  }

  // Finish table
  $header = array(t('Form ID'), t('Type'), t('Status'), t('(Edit)'));

  $output = theme('table', $header, $rows, array('id' => 'salesforce-admin-forms'));

  $output .= drupal_render($form);

  return $output;
}


/**
 * Process Salesforce forms administration form submission.
 */
function salesforce_admin_forms_submit($form_id, $form_values) {
  foreach ($form_values as $id => $sf) {
    // make reasonably certain we're dealing with salesforce form info
    if (is_array($sf) && isset($sf['status']) && isset($sf['type'])) {
      salesforce_expose_form($id, $sf, TRUE);  
    }
  }
  drupal_set_message(t("The Salesforce-exposed forms' settings have been updated."));
//  cache_clear_all();
}


/**
 * Handle menu callback for Salesforce form configuration.
 *
 * Takes a form_id passed in as an extra argument.
 */
function salesforce_admin_configure_form_page($form_id = NULL) {
  $output = '';
  if ($form_id && in_array($form_id, array_keys(salesforce_list_forms()))) { 
    // let different modules 
    $scf = module_invoke_all('salesforce_configure_forms', $form_id);
    if ($num = count($scf)) {
      // taking the top-weighted module makes more sense than mixing them
      $output .= $scf[0];
      if ($dif = $num - 1) {
        drupal_set_message(format_plural($dif, 'Note: there is one additional enabled module available to configure this form, but overridden.', 'Note: there are @count additional enabled modules available to configure this form, but overridden.'));
      }      
    }
    else {
      $output .= '<p>'. t('No Salesforce form-configuring modules found.') .'</p>';
      drupal_set_message(t('You do not have any modules enabled that can configure forms.  Enable a Salesforce-extending module such as the Salesforce fieldmatcher module.'));
    }
  }
  else {
    // redirect to main form list page
    drupal_set_message($form_id);
    $output .= drupal_get_form('salesforce_admin_forms');
  }
  return $output;
}


/**
 *  delete or disable a logged item
 */
function salesforce_log_manage_page() {
  $op = arg(3);
  $sid = arg(4);

  if (is_numeric($sid)) {
    switch ($op) {
      case 'disable':
        $disable = true;
        $message = t('disabled');
        break;
      case 'enable':
        $enable = true;
        $disable = false;
        $message = t('enabled');
        break;
      default:
        $disable = false;
        $message = t('deleted');
        break;
    }

    _salesforce_log_manage($sid, $disable, $enable);
    if (user_access(ADMINISTER_SALESFORCE)) {
      drupal_set_message(t('%message log entry', array('%message' => $message)));
    }
    drupal_goto('admin/logs/salesforce');
  }
}

/**
 *  run a logged cron task
 */
function salesforce_log_run_page() {
  $sid = arg(5);

  if (is_numeric($sid)) {
    $result = _salesforce_cron_run($sid);
    if ($result) {
      drupal_set_message(t('the logged task completed successfully'));
    } else {
      drupal_set_message(t('the logged task failed to complete'), 'error');
    }
  }

  drupal_goto('admin/logs/salesforce/logs');
}

/**
 *  helper functions
 */

function _salesforce_admin_list($op) {
  $sf = 'https://na1.salesforce.com/';

  if ($op == 'lead' || $op == 'contact') {
    $result = db_query("SELECT s.uid FROM {salesforce_users} s WHERE %s != ''", 's.'. $op .'_id');
    if (db_num_rows($result) > 0) {
      $cols = array(array('data' => t('User'), 'class' => 'user'), array('data' => t('Name')), array('data' => t('%i', array('%i' => ucfirst($op) .' ID')), 'class' => 'id'));
      if ($op == 'contact') {
        $cols[] = array('data' => t('Account ID', array('%i' => ucfirst($op))), 'class' => 'id');
      }

      while ($row = db_fetch_object($result)) {
        if ($op == 'contact') {
          // we do this incase we don't have a cached account id
          salesforce_account_select(user_load(array('uid' => $row->uid)));
        }
        $account = user_load(array('uid' => $row->uid));
        // remove the extra 3 chars that are not used in this case
        $link = $sf . substr($account->salesforce[$op.'_id'], 0, strlen($account->salesforce[$op.'_id']) - 3);
        $rows[$row->uid] = array(theme('username', $account), $account->first_name .' '. $account->last_name, l($account->salesforce[$op.'_id'], $link));
        if ($op == 'contact') {
          if ($account->salesforce['account_id']) {
            $account_link = $sf . substr($account->salesforce['account_id'], 0, strlen($account->salesforce['account_id']) - 3);
            $account_label = l($account->salesforce['account_id'], $account_link);
          } else {
            $account_label = t('N/A');
          }
          $rows[$row->uid][] = $account_label;
        }
      }

      $output .= '<p>'. t('%type in salesforce', array('%type' => ucfirst($op) .'s')) .'</p>'. "\n";
      $output .= theme('table', $cols, $rows, array('class' => 'salesforce_'. $op .'_list'));
    } else {
      drupal_set_message(t('currently no %op', array('%op' => $op .'s')));
    }
  }

  return $output;
}